/** 
 * <a href="http://servicemix.org">ServiceMix: The open source ESB</a> 
 * 
 * Copyright 2005 RAJD Consultancy Ltd
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 **/

package org.servicemix.jbi.nmr.flow;
import javax.jbi.JBIException;
import javax.jbi.management.LifeCycleMBean;
import javax.jbi.messaging.MessagingException;
import javax.management.JMException;
import javax.management.MBeanAttributeInfo;
import javax.management.ObjectName;
import org.servicemix.jbi.framework.ComponentNameSpace;
import org.servicemix.jbi.framework.LocalComponentConnector;
import org.servicemix.jbi.management.AttributeInfoHelper;
import org.servicemix.jbi.management.BaseLifeCycle;
import org.servicemix.jbi.messaging.ExchangePacket;
import org.servicemix.jbi.nmr.Broker;

/**
 * A simple Straight through flow
 * 
 * @version $Revision: 1.7 $
 */
public abstract class AbstractFlow extends BaseLifeCycle implements Flow {
    protected Broker broker;

    /**
     * Initialize the Region
     * 
     * @param broker
     * @throws JBIException
     */
    public void init(Broker broker) throws JBIException {
        this.broker = broker;
//      register self with the management context
        
        ObjectName objectName = broker.getManagementContext().createObjectName(this);
        try {
            broker.getManagementContext().registerMBean(objectName, this, LifeCycleMBean.class);
        }
        catch (JMException e) {
            throw new JBIException("Failed to register MBean with the ManagementContext", e);
        }
    }
    
    /**
     * start the flow
     * @throws JBIException
     */
    public void start() throws JBIException{
        super.start();
    }
    
    
    /**
     * stop the flow
     * @throws JBIException
     */
    public void stop() throws JBIException{  
        super.stop();
    }
    
    /**
     * shutDown the flow
     * @throws JBIException
     */
    public void shutDown() throws JBIException{
        super.shutDown();
        
    }

    /**
     * Distribute an ExchangePacket
     * 
     * @param packet
     * @throws MessagingException
     */
    public void doRouting(ExchangePacket packet) throws MessagingException {
        ComponentNameSpace id = packet.isOutbound() ? packet.getDestinationId() : packet.getSourceId();
        LocalComponentConnector lcc = broker.getRegistry().getLocalComponentConnector(id);
        if (lcc != null) {
            lcc.getDeliveryChannel().processInBound(packet);
        }
        else {
            throw new MessagingException("No component with id (" + id + ") - Couldn't route ExchangePacket " + packet);
        }
    }
    
    /**
     * Get an array of MBeanAttributeInfo
     * 
     * @return array of AttributeInfos
     * @throws JMException
     */
    public MBeanAttributeInfo[] getAttributeInfos() throws JMException {
        AttributeInfoHelper helper = new AttributeInfoHelper();
        helper.addAttribute(getObjectToManage(), "description", "The type of flow");
        return AttributeInfoHelper.join(super.getAttributeInfos(), helper.getAttributeInfos());
    }
}